/*
 *  Copyright (C) 1999, 2000  Heikki Tauriainen <Heikki.Tauriainen@hut.fi>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "SinglePathIterator.h"

namespace Graph
{

SinglePathIterator::SinglePathIterator
  (unsigned long int propositions_per_state,
   StateSpace::size_type number_of_states) :
  path(propositions_per_state, number_of_states), loop_target_state(0)
{
  StateSpace::size_type state;

  for (state = 0; (state + 1) < number_of_states; state++)
    path[state].connectTo(state + 1);

  path[state].connectTo(0);
}

bool SinglePathIterator::operator==(const SinglePathIterator& it) const
{
  if (loop_target_state != it.loop_target_state
      || path.size() != it.path.size())
    return false;

  StateSpace::size_type state;

  for (state = 0;
       state < path.size() && path[state].positiveAtoms()
	                        == it.path[state].positiveAtoms();
       state++)
    ;

  return (state == path.size());
}

bool SinglePathIterator::operator!=(const SinglePathIterator& it) const
{
  if (loop_target_state != it.loop_target_state
      || path.size() != it.path.size())
    return true;

  StateSpace::size_type state;

  for (state = 0;
       state < path.size() && path[state].positiveAtoms()
	                        == it.path[state].positiveAtoms();
       state++)
    ;

  return (state != path.size());
}

void SinglePathIterator::computeNextPath()
{
  if (atEnd())
    return;

  StateSpace::size_type state;

  for (state = 0;
       state < path.size() && path[state].positiveAtoms().count()
 	                        == path[state].positiveAtoms().size();
       state++)
    path[state].positiveAtoms().clear();

  if (state == path.size())
  {
    for (state = 0; state < path.size(); state++)
      path[state].positiveAtoms().clear();

    path[path.size() - 1].disconnectFrom(loop_target_state);

    if (loop_target_state < path.size())
      loop_target_state++;

    if (loop_target_state < path.size())
      path[path.size() - 1].connectTo(loop_target_state);
  }
  else
  {
    Bitset* truth_assignment = &path[state].positiveAtoms();
    unsigned long int proposition;

    for (proposition = 0; truth_assignment->test(proposition); proposition++)
      truth_assignment->clear(proposition);

    truth_assignment->set(proposition);
  }
}

}
