/*
 *  Copyright (C) 1999, 2000  Heikki Tauriainen <Heikki.Tauriainen@hut.fi>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _SinglePathIterator_h_
#define _SinglePathIterator_h_

#include "Bitset.h"
#include "StateSpace.h"

using namespace std;

namespace Graph
{

class SinglePathIterator
{
public:
  SinglePathIterator
    (unsigned long int propositions_per_state,
     StateSpace::size_type number_of_states);

  /* default copy constructor */

  ~SinglePathIterator();

  /* default assignment operator */

  bool operator==(const SinglePathIterator& it) const;

  bool operator!=(const SinglePathIterator& it) const;

  //bool operator<(const SinglePathIterator& it) const;

  //bool operator<=(const SinglePathIterator& it) const;

  //bool operator>(const SinglePathIterator& it) const;

  //bool operator>=(const SinglePathIterator& it) const;

  const StateSpace& operator*() const;
  const StateSpace* operator->() const;

  const StateSpace& operator++();
  const StateSpace operator++(int);

  bool atEnd() const;

private:
  void computeNextPath();

  StateSpace path;
  StateSpace::size_type loop_target_state;
};

inline SinglePathIterator::~SinglePathIterator()
{
}

inline const StateSpace& SinglePathIterator::operator*() const
{
  return path;
}

inline const StateSpace* SinglePathIterator::operator->() const
{
  return &path;
}

inline const StateSpace& SinglePathIterator::operator++()
{
  computeNextPath();
  return path;
}

inline const StateSpace SinglePathIterator::operator++(int)
{
  StateSpace old_path(path);
  computeNextPath();
  return old_path;
}

inline bool SinglePathIterator::atEnd() const
{
  return (loop_target_state == path.size());
}
 
}
#endif  /* _SinglePathIterator_h_ */
