/*
 *  Copyright (C) 1999, 2000  Heikki Tauriainen <Heikki.Tauriainen@hut.fi>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _TestRoundInfo_h_
#define _TestRoundInfo_h_

#include "BuchiAutomaton.h"
#include "FormulaString.h"
#include "ProductAutomaton.h"
#include "StateSpace.h"

using namespace std;

/******************************************************************************
 *
 * A data structure for storing test round control information and some test-
 * related data.
 *
 *****************************************************************************/

class TestRoundInfo
{
public:
  explicit TestRoundInfo();                         /* Constructor. */

  ~TestRoundInfo();                                 /* Destructor. */

  unsigned long int current_round;                  /* Number of current round.
						     */

  unsigned long int next_round_to_run;              /* Indicates the next
						     * round to run.
						     */

  unsigned long int next_round_to_stop;             /* Indicates the next
						     * test round after which
						     * to pause and wait for
						     * user commands.
						     */

  bool error;                                       /* True if an error
						     * occurred during the
						     * current round.
						     */

  bool skip;                                        /* True if the current
						     * round is to be skipped.
						     */

  bool abort;                                       /* True if the testing is
						     * to be aborted.
						     */
  
  const Graph::StateSpace* statespace;              /* Pointer to the
						     * reachability graph used
						     * in the current test
						     * round.
						     */

  const Graph::BuchiAutomaton* buchi_automaton;     /* Pointer to the Bchi
						     * automaton used in the
						     * current test round.
						     */

  const Graph::ProductAutomaton* product_automaton; /* Pointer to the product
						     * automaton used in the
						     * current test round.
						     */

  unsigned long int                                 /* Indicates the number  */
    number_of_rounds_to_change_statespace;          /* of rounds after which
						     * a new reachability
						     * graph is generated.
						     */

  unsigned long int                                 /* Indicates the number  */
    number_of_rounds_to_change_formula;             /* of rounds after which
						     * a new formula is
						     * generated.
						     */

  FormulaString::FormulaString formulae[4];         /* Formulae used in the
						     * current round.
						     */

  char formula_file_name[L_tmpnam + 1];             /* Names for two        */
  char negated_formula_file_name[L_tmpnam + 1];     /* temporary files
						     * used for passing the
						     * formulae to the
						     * different LTL->Bchi
						     * converter
						     * implementations.
						     */

private:
  TestRoundInfo(const TestRoundInfo& info);         /* Copy constructor (hidden
						     * from external usage).
						     */

  TestRoundInfo& operator=                          /* Assignment operator   */
    (const TestRoundInfo& info);		    /* (hidden from external
						     * usage).
						     */
};



/******************************************************************************
 *
 * Inline function definitions for class TestRoundInfo.
 *
 *****************************************************************************/

/* ========================================================================= */
inline TestRoundInfo::TestRoundInfo()
/* ----------------------------------------------------------------------------
 *
 * Description:   Constructor for class TestRoundInfo. Creates a new
 *                TestRoundInfo object.
 *
 * Arguments:     None.
 *
 * Returns:       Nothing.
 *
 * ------------------------------------------------------------------------- */
{
}

/* ========================================================================= */
inline TestRoundInfo::~TestRoundInfo()
/* ----------------------------------------------------------------------------
 *
 * Description:   Destructor for class TestRoundInfo.
 *
 * Arguments:     None.
 *
 * Returns:       Nothing.
 *
 * ------------------------------------------------------------------------- */
{
}



#endif  /* _TestRoundInfo_h_ */
