/*
 *  Copyright (C) 1999, 2000  Heikki Tauriainen <Heikki.Tauriainen@hut.fi>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _prod_lbt_h_
#define _prod_lbt_h_

#include <cstdio>
#include <cstring>
#include <deque>
#include <fstream>
#include <set>
#include <stack>
#include <string>
#include "Exception.h"
#include "FormulaString.h"
#include "NeverClaimParser.h"
#include "StlConfig.h"

using namespace std;

namespace prod_buchi
{

using FormulaString::FormulaString;

char PROD_TRUE[]        = "1";
char PROD_FALSE[]       = "0";
char PROD_NOT[]         = "not";
char PROD_FINALLY[]     = "eventually";
char PROD_GLOBALLY[]    = "henceforth";
char PROD_AND[]         = "and";
char PROD_OR[]          = "or";
char PROD_IMPLICATION[] = "implies";
char PROD_UNTIL[]       = "until";

class TranslateAtom
{
public:
  static void eval
    (FormulaString::TraversalInfo<string*>& info,
     const string& atom);

  static set<string, less<string>, ALLOC(string) > atoms;
};

set<string, less<string>, ALLOC(string) > TranslateAtom::atoms;

template<const char* C>
class TranslateConstant
{
public:
  static void eval(FormulaString::TraversalInfo<string*>& info);
};

template<const char* C>
class TranslateUnary
{
public:
  static void eval(FormulaString::TraversalInfo<string*>& info);
};

template<const char* C>
class TranslateBinary
{
public:
  static void eval(FormulaString::TraversalInfo<string*>& info);
};

class TranslateV
{
public:
  static void eval(FormulaString::TraversalInfo<string*>& info);

private:
  friend class TranslateEquivalence;
  static bool warned_about_operator_expansion;
};

bool TranslateV::warned_about_operator_expansion = false;

class TranslateEquivalence
{
public:
  static void eval(FormulaString::TraversalInfo<string*>& info);
  static stack<string*, deque<string*, ALLOC(string*) > > operand_stack;
};

stack<string*, deque<string*, ALLOC(string*) > >
  TranslateEquivalence::operand_stack;

class TranslateError
{
public:
  static void eval(FormulaString::TraversalInfo<string*>& info);
};

}

#endif  /* _prod_lbt_h_
